/****************************************************************
 * Copyright (C)2000 - Just van den Broecke - See license below *
 ****************************************************************/

package nl.justobjects.toolkit.sys;


/**
 * Terminates Threads.
 *
 * <h3>Purpose and Responsibilities</h3>
 *
 * In many cases it is hard to stop Threads. Thread.stop() is deprecated and dangerous to
 * use. However in some cases like when a Thread blocks for I/O it is really hard to
 * kill the Trhead. The method terminate() will try to stop a Thread in a sequence
 * of steps, taking more and more hard measures to kill the beast.
 * (a different version will be called Terminator2 :-).
 *
 * Credits: code from "Concurrent Programming in Java 2nd ed" by Doug Lea.
 *
 * @author   Just van den Broecke
 * @version $Id: ThreadUtil.java,v 1.3 2003/01/06 00:23:49 just Exp $
 */
public class ThreadUtil {


	/**
	 * Try to kill a Thread, return true if known to be dead.
	 */
	public static boolean terminate(Thread aThread, long aMaxWaitToDieMillis) {
		if (aThread == null) {
			// Sick case but ok.
			return true;
		}

		if (!aThread.isAlive()) {
			// Already dead.
			return true;
		}

		// Phase 1: graceful cancellation
		aThread.interrupt();
		try {
			aThread.join(aMaxWaitToDieMillis);
		} catch (InterruptedException ignore) {
		}

		if (!aThread.isAlive()) {
			// Success.
			return true;
		}

		// Phase 2: brute force
		aThread.stop();

		if (!aThread.isAlive()) {
			// Success.
			return true;
		}

		// Phase 3: minimize damage
		aThread.setPriority(Thread.MIN_PRIORITY);

		// Hmm, nothing worked.
		return false;
	}
}

/*
 * $Log: ThreadUtil.java,v $
 * Revision 1.3  2003/01/06 00:23:49  just
 * moved devenv to linux
 *
 * Revision 1.2  2002/11/14 20:25:19  just
 * reformat of code only
 *
 * Revision 1.1  2002/10/04 09:25:52  just
 * rename to ThreadUtil (name clash with java.lang)
 *
 * Revision 1.1  2002/10/04 09:20:45  just
 * no message
 *
 *
 *
 */


/*
   Copyright (C)2000 Just A. van den Broecke <just@justobjects.nl>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
