/****************************************************************
 * Copyright (C)2000 - Just van den Broecke - See license below *
 ****************************************************************/
package nl.justobjects.toolkit.xml;

import org.w3c.dom.Document;
import org.xml.sax.InputSource;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;

/**
 * Wraps the Apache Xerces parser.
 * $Id: JAXPDOMParser.java,v 1.1 2003/11/19 15:16:51 just Exp $
 * @author $Author: just $ - Just van den Broecke - Just Objects B.V. &copy;
 */
public class JAXPDOMParser extends nl.justobjects.toolkit.xml.DOMParser {
	/** Parser. */
	private static DocumentBuilderFactory documentBuilderFactory;
	private DocumentBuilder documentBuilder;
	private static final String DOC_BUILDER_FACTORY = "org.apache.xerces.jaxp.DocumentBuilderFactoryImpl";

	static {
		// This is the only place we are Xerces-specific...
		// documentBuilderFactory = DocumentBuilderFactory.newInstance();
		// System.setProperty("javax.xml.parsers.DocumentBuilderFactory", "org.apache.xerces.jaxp.DocumentBuilderFactoryImpl");
		// System.setProperty("javax.xml.parsers.SAXParserFactory", "org.apache.xerces.jaxp.SAXParserFactoryImpl");
		try {
			documentBuilderFactory = (DocumentBuilderFactory) Class.forName(DOC_BUILDER_FACTORY).newInstance();
			System.out.println("JAXPDOMParser using " + documentBuilderFactory.getClass().getName());
		} catch (Throwable t) {
			System.err.println("ERROR: cannot find class " + DOC_BUILDER_FACTORY + " (is xerces in your classpath?)");
		}
	}

	/** Default constructor. */
	public JAXPDOMParser() {
		try {
			documentBuilderFactory.setValidating(false);
			documentBuilderFactory.setNamespaceAware(true);
			documentBuilder = documentBuilderFactory.newDocumentBuilder();
			documentBuilder.setErrorHandler(this);
		} catch (Exception e) {
			p("ERROR: in constructor e=" + e);
		}
	}

	/** Parses the specified URI and returns the document. */
	public Document parse(String uri) throws DOMParserException {
		if (documentBuilder == null) {
			throw new DOMParserException("Error creating DocumentBuilder");
		}

		try {
			p("parsing: " + uri);
			// See if we have an entity resolver
			if (getEntityResolver() != null) {
				// p("parsing with EntityResolver " + uri);

				// Ok. Assign to parser for entitites within the document
				// like DTDs.
				documentBuilder.setEntityResolver(getEntityResolver());
				//saxParserFactory.setFeature("http://xml.org/sax/features/use-entity-resolver2", true);

				try {
					// Also resolve our URI (e.g. a cpath:/)
					InputSource is = getEntityResolver().resolveEntity(null, uri);

					if (is == null) {
						throw new DOMParserException("could not resolve uri=" + uri);
					}
					// is.setSystemId("/");
					return documentBuilder.parse(is);
				} catch (Exception e) {
					e.printStackTrace();
					throw new DOMParserException(e);
				}
			} else {
				// No entity resolver: just call parser with URI
				try {
					return documentBuilder.parse(uri);
				} catch (Exception e) {
					throw new DOMParserException(e);
				}
			}
		} catch (Exception e) {
			throw new DOMParserException("Error parsing document ", e);
		}
	}

	public void setProperty(String key, String value) throws DOMParserException {
		if ("validate".equals(key) && "true".equals(value)) {
			try {
				documentBuilderFactory.setValidating(true);
			} catch (Exception e) {
				throw new DOMParserException(e);
			}
		}
	}

	public static void p(String s) {
		// System.out.println("JAXPDOMParser: " + s);
	}
}

/*
 * $Log: JAXPDOMParser.java,v $
 * Revision 1.1  2003/11/19 15:16:51  just
 * new
 *
 *
 */

/*
   Copyright (C)2000 Just A. van den Broecke <just@justobjects.nl>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
