/****************************************************************
 * Copyright (C)2000 - Just van den Broecke - See license below *
 ****************************************************************/

package nl.justobjects.xbook;

import nl.justobjects.toolkit.io.IO;
import nl.justobjects.toolkit.xml.DOMUtil;
import nl.justobjects.toolkit.xml.Query;
import nl.justobjects.toolkit.xml.XML2HTML;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import java.io.File;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

/**
 * Generates menu-driven HTML documents from xbook XML file.
 * <p>
 * <b>Purpose</b><br>
 * TBS.
 * <p>
 * <b>Implementation</b><br>
 * TBS.
 * <p>
 * <b>Examples</b><br>
 * Give some code examples. Be sure to use the &lt;pre&gt; HTML tag.
 * <p>
 * <br>
 * $Source: /var/cvs/justobjects/xbook/src/nl/justobjects/xbook/XBook.java,v $<p>
 *
 * @version $Revision: 1.24 $
 * @author $Author: just $ - Just van den Broecke - Just Objects &copy;
 * @since $Date: 2003/11/19 15:12:09 $
 * @see <package name.classname#method>
 */
public class XBook {

	/** Stylesheet to transform a book.xml to menu.xml for final site generation. */
	static final private String BOOK_TO_MENU_XSL = "nl/justobjects/xbook/rsc/xsl/book2menu.xsl";

	static public void generate(String aBookFile, String aDestinationDir, boolean lite) throws Exception {
		p("XBook " + Version.SOFTWARE_VERSION + " (built " + Version.BUILD_DATE + ")");
		p("start book= " + aBookFile + " dest=" + aDestinationDir);
		long startTime = System.currentTimeMillis();

		// Validate files and dirs, create if neccessary
		File bookFile = new File(aBookFile);
		if (!bookFile.exists()) {
			throw new Exception(aBookFile + " does not exist");
		}

		// The directory of the book XML file.
		String bookFileDir = bookFile.getParent();
		if (bookFileDir == null) {
			bookFileDir = ".";
		}

		// Create destination dir if required
		File destinationDir = new File(aDestinationDir);
		if (!destinationDir.exists()) {
			IO.mkdir(aDestinationDir);
		}

		String destinationDocumentDir = aDestinationDir;

		// Parse the book file
		p("parsing " + aBookFile);
		Document bookDocument = DOMUtil.parse(aBookFile, true);


		//

		// Generate HTML for all XML documents using XSL
		//

		// Assemble all assets dirs here as (source, destination) pairs.
		Hashtable assetsDirs = new Hashtable(3);
		String assetsSrcDir = bookFileDir + File.separator + "assets";

		// Root source dir assets
		if (new File(assetsSrcDir).exists()) {
			assetsDirs.put(assetsSrcDir, destinationDocumentDir + File.separator + "assets");
		}

		// Get all 'document' tags
		Query query = new Query(bookDocument);
		query.addMatcherByTag("document");
		Vector result = query.query();
		p("processing " + result.size() + " documents with xdoc");

		// Go through all documents (<document> elements) , transforming each to HTML using XSL
		for (int i = 0; i < result.size(); i++) {
			Element documentElement = (Element) result.elementAt(i);

			// Create the relative and full file paths for the source document.
			String sourceDocumentFileName = documentElement.getAttribute("src");
			String sourceDocumentFilePath = bookFileDir + File.separator + sourceDocumentFileName;

			// If no "url" attribute specified, generate the destination file from the source file.
			String destinationDocumentFile = documentElement.getAttribute("url");
			if (destinationDocumentFile == null || destinationDocumentFile.length() == 0) {
				destinationDocumentFile = sourceDocumentFileName.substring(0, sourceDocumentFileName.indexOf(".xml")) + ".html";
			}

			// Create the full file path for the destination document.
			String destinationDocumentFilePath = destinationDocumentDir + "/" + destinationDocumentFile;

			// Assign to url attribute (used in links later on)
			documentElement.setAttribute("url", destinationDocumentFile);

			// Let XDoc do the transformation work for each source/destination file.
			XDoc xDoc = new XDoc();
			xDoc.setSourceFilePath(sourceDocumentFilePath);
			xDoc.setDestinationFilePath(destinationDocumentFilePath);
			xDoc.transform();

			// If document has an "assets" subdir and not yet in table add it.
			String sourceAssetsDir = xDoc.getAssetsDir();
			if (sourceAssetsDir != null) {
				String destinationAssetsDir = new File(destinationDocumentFilePath).getParent() + File.separator + "assets";
				assetsDirs.put(sourceAssetsDir, destinationAssetsDir);
			}
		}

		// If the default page is an XML file process it with XDoc and replace the name with
		// the resulting .html file
		Element bookDocumentElement = bookDocument.getDocumentElement();
		String defaultPageFileName = bookDocumentElement.getAttribute("default");
		if (defaultPageFileName != null && defaultPageFileName.length() != 0) {
			String defaultDocumentFilePath = bookFileDir + File.separator + defaultPageFileName;
			String destinationDocumentFile = defaultPageFileName;
			String destinationDocumentFilePath = destinationDocumentDir + File.separator + destinationDocumentFile;
			if (defaultPageFileName.endsWith(".xml")) {
				destinationDocumentFile = defaultPageFileName.substring(0, defaultPageFileName.indexOf(".xml")) + ".html";
				destinationDocumentFilePath = destinationDocumentDir + File.separator + destinationDocumentFile;
				XDoc xDoc = new XDoc();
				xDoc.setSourceFilePath(defaultDocumentFilePath);
				xDoc.setDestinationFilePath(destinationDocumentFilePath);
				xDoc.transform();
			}

			bookDocumentElement.setAttribute("default", destinationDocumentFile);
		}

		//
		// Process external docs if required
		//

		// Get all <external/> tags
		Query externalQuery = new Query(bookDocument);
		externalQuery.addMatcherByTag("external");
		Vector externalQueryResult = externalQuery.query();
		p("processing " + result.size() + " external documents");

		// Copy external documents to destination if copy attr == "y(es)"
		for (int i = 0; i < externalQueryResult.size(); i++) {
			// get next "external" node
			Element externalDocumentElement = (Element) externalQueryResult.elementAt(i);

			// Check for "copy" attr and copy if required
			String copyAttr = externalDocumentElement.getAttribute("copy");
			if (copyAttr.startsWith("y")) {

				// Get source file name
				String fileAttr = externalDocumentElement.getAttribute("url");
				String sourceDocumentFile = bookFileDir + File.separator + fileAttr;
				if (!new File(sourceDocumentFile).exists()) {
					throw new Exception("external document " + sourceDocumentFile + " does not exist");
				}

				// Create destination file name
				String destinationDocumentFile = destinationDir + File.separator + fileAttr;

				// Copy, possibly creating destination directories
				p("copy external " + sourceDocumentFile + " to " + destinationDocumentFile);
				IO.createFilePath(destinationDocumentFile);
				IO.cp(sourceDocumentFile, destinationDocumentFile);

				// If assets subdir and not yet in table add it.
				String sourceAssetsDir = new File(sourceDocumentFile).getParent() + File.separator + "assets";
				if (new File(sourceAssetsDir).exists()) {
					String destinationAssetsDir = new File(destinationDocumentFile).getParent() + File.separator + "assets";
					assetsDirs.put(sourceAssetsDir, destinationAssetsDir);
				}
			}
		}

		// Copy assembled assets directories if present
		p("Copying assets...");
		for (Enumeration e = assetsDirs.keys(); e.hasMoreElements();) {
			String sourceAssetsDir = (String) e.nextElement();
			String destinationAssetsDir = (String) assetsDirs.get(sourceAssetsDir);
			String assetsDestDir = destinationDocumentDir + File.separator + "assets";
			IO.cpdir(sourceAssetsDir, destinationAssetsDir);
			p("sourceAssetsDir=" + sourceAssetsDir + " destinationAssetsDir=" + destinationAssetsDir);
		}


		// Generate the site using XMenu tools
		//
		String xmenuFile = bookFileDir + File.separator + "xmenu.xml";

		// Generate XMenu XML file
		p("Generating site using XMenu...");
		XML2HTML.xml2Html(bookDocument, BOOK_TO_MENU_XSL, xmenuFile);
		//

		// Generate the entire site using XMenu
		XMenu.generate(xmenuFile, aDestinationDir, lite);
		IO.rm(xmenuFile);

		p("OK - all done in " + (System.currentTimeMillis() - startTime) / 1000 + " seconds");
	}


	static private void p(String s) {
		System.out.println("XBook: " + s);
	}

	static private void fatal(String what, Exception e) {
		p("ERROR - " + what + " exception =" + e);
		e.printStackTrace();
		System.exit(-1);
	}

	static public void main(String args[]) {
		String bookXML = null;
		String destDir = null;
		boolean lite = false;

		// check for -lite XBook option (lightweight HTML menu)
		if (args.length == 3) {
			if (!"-lite".equals(args[0])) {
				p("-lite is the only option with three args");
				System.exit(0);
			} else {
				lite = true;
			}
			bookXML = args[1];
			destDir = args[2];
		} else if (args.length == 2) {
			bookXML = args[0];
			destDir = args[1];
		} else {
			p("Usage: xbook [-lite] <xbook.xml> <target site dir>");
			System.exit(0);
		}

		try {
			// Generate the book
			generate(bookXML, destDir, lite);
		} catch (Exception e) {
			fatal("Exception in generate()", e);
		}

		// Somehow the system may hang when XAlbum is used (something in ImageIcon holding threads?).
		// So this uses an explicit exit.
		System.exit(0);
	}
}

/*
 * $Log: XBook.java,v $
 * Revision 1.24  2003/11/19 15:12:09  just
 * many changes for invoking xbook in ant
 *
 * Revision 1.23  2003/01/06 10:27:22  just
 * removed Arial fonts from default styles
 *
 * Revision 1.22  2002/07/28 11:57:49  just
 * getAssetsDir() returned wrong file name in XDoc
 *
 * Revision 1.21  2002/07/11 12:02:52  just
 * fix generate() fail when default doc is not XML
 *
 * Revision 1.20  2002/07/06 15:57:28  just
 * updates for processing a default XML document for the menu
 *
 * Revision 1.19  2002/07/04 14:09:23  just
 * mainly comments
 *
 * Revision 1.18  2002/07/04 12:50:50  just
 * no message
 *
 * Revision 1.17  2002/07/03 21:59:37  just
 * introduce common XDoc class for all document tags
 *
 * Revision 1.16  2002/07/02 21:41:38  just
 * no message
 *
 * Revision 1.15  2002/07/01 22:04:17  just
 * updates for refactoring
 *
 * Revision 1.14  2002/06/18 11:04:11  just
 * support for URL attribute in document
 *
 * Revision 1.13  2002/06/18 10:08:04  just
 * exit in main and document  DTD checking based on xhtml presence
 *
 * Revision 1.12  2002/06/17 15:11:14  just
 * added album as document task
 *
 * Revision 1.11  2002/04/28 12:48:50  just
 * xmenu integration in xbook
 *
 * Revision 1.10  2002/04/28 11:54:46  just
 * xmenu integration in xbook
 *
 * Revision 1.9  2002/03/27 20:08:26  just
 * support for html escape with xhtml tag in text.dtd and text2html.xsl
 *
 * Revision 1.8  2002/02/04 23:27:40  just
 * mainly changes to find resources through CLASSPATH (no cpath:/ anymore)
 *
 * Revision 1.7  2002/01/24 01:15:07  just
 * various updates (see version 1.0.5 info)
 *
 * Revision 1.6  2002/01/10 15:18:22  just
 * make source path absolute when copying external docs
 *
 * Revision 1.5  2002/01/03 15:32:11  just
 * changes required for upgrade to Xalan 2.2D14
 *
 * Revision 1.4  2001/12/17 23:52:15  just
 * lite option and copy of default file
 *
 * Revision 1.3  2001/12/13 23:12:54  just
 * no real changes
 *
 * Revision 1.2  2001/12/13 10:13:27  just
 * various changes
 *
 * Revision 1.1  2001/12/11 16:26:24  just
 * new
 *
 * Revision 1.7  2001/02/11 20:37:33  kstroke
 * fixed copying from resources (css file)
 *
 * Revision 1.6  2001/02/11 20:32:16  kstroke
 * fixed copying from resources (html+icons)
 *
 * Revision 1.5  2001/02/11 17:10:19  kstroke
 * removed cpdir for HTML and icons: copy files to enable xmenu using from .jar
 *
 * Revision 1.4  2000/12/28 15:55:45  just
 * bugfix properties init (bgColor typo)
 *
 * Revision 1.3  2000/12/17 17:00:44  just
 * bug fixes
 *
 * Revision 1.2  2000/12/11 23:53:06  just
 * modified fgcolor/bgcolor setting
 *
 * Revision 1.1  2000/12/09 16:36:58  just
 * new
 *
 */

/*
   Copyright (C)2000 Just A. van den Broecke <just@justobjects.nl>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/